//---------------------------------------------------------------------
// SPACE TAPPER: game.h
// by Omar Cornut
//---------------------------------------------------------------------
// Game Engine
//---------------------------------------------------------------------

//---------------------------------------------------------------------

void	game_main_start(va_list args);		// Parameter(s): none
void    game_main_restart(void);
void	game_main_next_level(void);
void	game_main_update(void);
void	game_main_stop(void);

//---------------------------------------------------------------------

#define GAME_UPDATE_ENTITIES	        	(0x01)
#define GAME_UPDATE_PLAYER	        		(0x02)
#define GAME_UPDATE_SEQUENCE        		(0x04)

#define GAME_UPDATE_NONE        			(0x00) /* PAUSE */
#define GAME_UPDATE_ALL			        	(GAME_UPDATE_ENTITIES | GAME_UPDATE_PLAYER | GAME_UPDATE_SEQUENCE)

#define GAME_LIVES_START        			(4)

#define GAME_DIFFICULTY_EASY				(0)
#define GAME_DIFFICULTY_NORMAL				(1)
#define GAME_DIFFICULTY_HARD				(2)
#define GAME_DIFFICULTY_MAX					(3)

#define GAME_SPECIAL_NONE					(0)
#define GAME_SPECIAL_ENTERTAINERS_1			(1)
#define GAME_SPECIAL_TUBEALIEN				(2)
#define GAME_SPECIAL_ENTERTAINERS_2			(3)

#define GAME_RULES_LOSE_BEER_FALL_LEFT      (0x01)
#define GAME_RULES_LOSE_BEER_FALL_RIGHT     (0x02)
#define GAME_RULES_LOSE_CUSTOMER_REACH_LEFT (0x04)
#define GAME_RULES_DEFAULT                  (GAME_RULES_LOSE_BEER_FALL_LEFT | GAME_RULES_LOSE_BEER_FALL_RIGHT | GAME_RULES_LOSE_CUSTOMER_REACH_LEFT)

#define GAME_BEER_CATCH_SCORE_INCREASE      (100)

// FIXME: sprite font palette ...
#define PAUSE_PALETTE_IDX                   (5)

//---------------------------------------------------------------------

// FIXME: sort out game/player stuff, it's a bit messy now

typedef struct
{
	// State
	t_game_state	state;
	
	// In-Game
	int             paused;
	int				update_flags;
	int				score_displayed;
	
	// Backgrounds
	t_bg			bg0;				// Layer 0: Layout
	t_bg			bg1;				// Layer 1: Bars
	t_bg			bg2;				// Layer 2: Entertainers & Misc
	t_bg			bg3;				// Layer 3: Interfaces
	int				font_tiles_start;
	
	// Blocks (FIXME: size_x/y in blocks should be optimally stored in ROM)
	t_bg_block		block_bar_start;	// Bar start GFX
	t_bg_block		block_bar_repeat;	// Bar repeat GFX
	union
	{
	    //t_bg_block  layout3_platform; // Moving platform GFX (layout 3)
        t_bg_block  layout5_part;       // Additionnal part GFX (layout 5)
	}               block_additionnal;
	
	// Inputs
	u16				keys;				// Keys input
	u16				keys_old;			// Keys input on previous frame
}					t_game;

t_game				game;

// Note: Accessing game.xxx does not add an overhead. 'game' is resolved at
// compilation, and reading/writing opcodes on the ARM all requires an offset.

//---------------------------------------------------------------------
