//---------------------------------------------------------------------
// SPACE TAPPER: game_entity_customers.h
// by Omar Cornut
//---------------------------------------------------------------------
// Game - Entity - Customers handling
//---------------------------------------------------------------------

// Customers Animations
#define CUSTOMER_ANIM_STANDING					(0)
#define CUSTOMER_ANIM_PICKING       			(1)
#define CUSTOMER_ANIM_DRINKING					(2)
#define CUSTOMER_ANIM_WATCHING					(3)
// For specific customers
#define CUSTOMER_ANIM_FAST_THROWER_THROWING_1   (4)
#define CUSTOMER_ANIM_FAST_THROWER_THROWING_2   (5)
#define CUSTOMER_ANIM_HEAVY_DRINKER_THROWING    (4)
#define CUSTOMER_ANIM_HEAVY_DRINKER_DRANK       (5)

// Customers Sprite informations
#define	CUSTOMER_SPR_OFFSET_BAR_Y				(-25)

// Miscellaneous
#define CUSTOMER_THIRSTYNESS        			(2)
#define CUSTOMER_SPEED_WALK						(FFIX4(0.5))
#define CUSTOMER_SPEED_THROWN_BACK				(FFIX4(1.0))
#define CUSTOMER_STEPS_BACK						(48)

#define CUSTOMER_HEAVY_DRINKER_THIRSTYNESS		(5)
#define CUSTOMER_HEAVY_DRINKER_STEPS_BACK		(24)

// Types
#define CUSTOMER_TYPE_FAST_THROWER				(3)
#define CUSTOMER_TYPE_HEAVY_DRINKER				(5)
#define	CUSTOMER_TYPE_MAX						(7)

// Customers States
#define CUSTOMER_STATE_NORMAL					(0)
#define CUSTOMER_STATE_DRINKING					(1)
#define CUSTOMER_STATE_WATCHING					(2)
#define CUSTOMER_STATE_OUT						(3)
#define CUSTOMER_STATE_THROWING                 (4)

// Customers Sub States
#define CUSTOMER_SUB_STATE_DRINKING_MOVE_BACK	(0)
#define CUSTOMER_SUB_STATE_DRINKING_ANIM_INIT	(1)
#define CUSTOMER_SUB_STATE_DRINKING_ANIM		(2)

//---------------------------------------------------------------------

void				customers_init_videodata(void);

//---------------------------------------------------------------------

// Customer definition (in ROM)
typedef struct
{
	s16				type;				// Alien type
	s16				palette_idx;		// Palette to use
	s16				tiles_data_size;	// Total size of all tiles
	s8				beer_shift_r;		// Delta for beer placement relative to center (right direction)
	s8				beer_shift_l;		// Delta for beer placement relative to center (left direction)
	int             drink_point_given;  // Number of points given by making this alien drink
	int             money_given;        // Money left
	void *		    palette_data;		// Pointer to palette in ROM
	void *			tiles_data;			// Pointer to tiles in ROM
	ts_anims_set *	anims_set;			// Pointer to animation set in ROM
}					ts_customer_def;

extern const ts_customer_def customers_def[CUSTOMER_TYPE_MAX];

//---------------------------------------------------------------------

t_entity *			customer_new(int bar_idx, int customer_type);
void				customer_delete(t_entity *c);
void				customer_update(t_entity *c);

void				customer_normal(t_entity *c);
void				customer_drink(t_entity *c, t_entity *beer);
void				customer_watch(t_entity *c);
void                customer_fast_thrower_throw(t_entity *c);
void                customer_heavy_drinker_throw(t_entity *c);

//---------------------------------------------------------------------
