//---------------------------------------------------------------------
// SPACE TAPPER: game_levels_sequencer.c
// by Omar Cornut
//---------------------------------------------------------------------
// Game - Levels Sequencer
//---------------------------------------------------------------------

#define SEQUENCER_INSTRUCTIONS_NEEDED
#include "tapper.h"

//---------------------------------------------------------------------
// DATA
//---------------------------------------------------------------------

t_level_sequencer	sequencer;
const u8			sequencer_instructions_size[SEQUENCE_MAX] =
{
	1 + 1,	// SEQUENCE_DELAY
	1 + 2,	// SEQUENCE_ALIEN
	1 + 2,	// SEQUENCE_VAR_SET
	1 + 1,	// SEQUENCE_LABEL_SET
	1 + 2,	// SEQUENCE_DJNZ
	1 + 1,	// SEQUENCE_ENTERTAIN
	1 + 2,	// SEQUENCE_WAIT_COUNT
	1		// SEQUENCE_END
};

//---------------------------------------------------------------------
// FUNCTIONS
//---------------------------------------------------------------------

void			level_sequencer_start(const u8 *sequence)
{
	int			i;
	
	sequencer.active = YES;
	sequencer.completed = NO;
	sequencer.delay = 0;
	sequencer.sequence = (u8 *)sequence;
	sequencer.ip = (u8 *)sequence + 0;
	for (i = 0; i < SEQUENCER_VAR_MAX; i++)
		sequencer.vars[i] = 0;
	for (i = 0; i < SEQUENCER_LABEL_MAX; i++)
		sequencer.labels[i] = 0;
}

void			level_sequencer_stop(void)
{
	sequencer.active = NO;
}

// FIXME: this function may be optimized
// It is necessary, however ?
static int		level_sequencer_get_range_value(u8 range)
{
	// If only one bit is set, return its position
	switch (range)
	{
		case 0x01: return (0);
		case 0x02: return (1);
		case 0x04: return (2);
		case 0x08: return (3);
		case 0x10: return (4);
		case 0x20: return (5);
		case 0x40: return (6);
		case 0x80: return (7);
	}
	// Count number of '1' bits in the range
	int count = 0;
	int choices[8];
	int i;
	for (i = 0; i < 8; i++)
		if (range & (1 << i))
			choices[count++] = i;
	// Pick a random value between 0 and count - 1
	int r = (count == 4) ? (rand() & 3) : (rand() % count); // FIXME: avoid using rand() and %
	// Return the value
	return (choices[r]);
}

void			level_sequencer_update(void)
{
	if (sequencer.active == NO)	
		return;
	if (sequencer.delay > 0)
	{
		sequencer.delay--;
		return;
	}
	
	int instr;
	int difficulty;
	int arg1, arg2;
	while (sequencer.active && sequencer.delay == 0)
	{
		// Check difficulty flags
		difficulty = *sequencer.ip++;
		if (difficulty != 0)
			if ((game.state.difficulty == GAME_DIFFICULTY_EASY   && !(difficulty & EASY))
			 || (game.state.difficulty == GAME_DIFFICULTY_NORMAL && !(difficulty & NORMAL))
			 || (game.state.difficulty == GAME_DIFFICULTY_HARD   && !(difficulty & HARD)))
			    {
					sequencer.ip += sequencer_instructions_size[*sequencer.ip];
					continue;
				}

		// Read and interpret instruction
		instr = *sequencer.ip++;
		switch (instr)
		{
			// 0x00 - SEQUENCE_DELAY ----------------------------------
			case SEQUENCE_DELAY:
				sequencer.delay = (*sequencer.ip++) * 6; 	// (6 = 1/10 of a second)
				break;
			// 0x01 - SEQUENCE_ALIEN ----------------------------------
			case SEQUENCE_ALIEN: // FIXME
				arg1 = level_sequencer_get_range_value(*sequencer.ip++);
				arg2 = level_sequencer_get_range_value(*sequencer.ip++);
				customer_new(arg1, arg2);
				break;
			// 0x02 - SEQUENCE_VAR_SET --------------------------------
			case SEQUENCE_VAR_SET:
				arg1 = *sequencer.ip++;
				sequencer.vars[arg1] = *sequencer.ip++;
				break;
			// 0x03 - SEQUENCE_LABEL_SET ------------------------------
			case SEQUENCE_LABEL_SET:
				arg1 = *sequencer.ip++;
				sequencer.labels[arg1] = sequencer.ip - sequencer.sequence;
				break;
			// 0x04 - SEQUENCE_DJNZ -----------------------------------
			case SEQUENCE_DJNZ:
				arg1 = *sequencer.ip++;
				arg2 = *sequencer.ip++;
				if (--sequencer.vars[arg1] != 0)
					sequencer.ip = sequencer.sequence + sequencer.labels[arg2]; 
				break;
			// 0x05 - SEQUENCE_ENTERTAIN ------------------------------
			case SEQUENCE_ENTERTAIN:
				arg1 = *sequencer.ip++;
				// if (game.state.level->special == GAME_SPECIAL_ENTERTAINERS_1)
				entertainers_active(arg1 * 6);
				break;	
			// 0x06 - SEQUENCE_WAIT_COUNT -----------------------------
			case SEQUENCE_WAIT_COUNT: // FIXME
				arg1 = *sequencer.ip++;
				arg2 = *sequencer.ip++;
				if (arg1 == BAR_ALL)
				{
					if (entities_counter[ENTITY_TYPE_CUSTOMER] <= arg2)
						break;
				}
				else
				{
					int i;
					for (i = 0; i < BAR_MAX; i++)
						if (arg1 & (1 << i))
							if ((arg2 -= bar_entities_count(i, ENTITY_TYPE_CUSTOMER)) < 0)
								break;
					if (arg2 >= 0)
						break;
				}
				// Else, loop on current instruction and exit from sequencer
				sequencer.ip -= 4;
				return;
			// 0x07 - SEQUENCE_END ------------------------------------
			case SEQUENCE_END:
				sequencer.active = NO;
				sequencer.completed = YES;
				break;
			default:
				fatal("Unknown SEQUENCE opcode");
				break;
		}
	}
}

//---------------------------------------------------------------------
