//---------------------------------------------------------------------
// SPACE TAPPER: game_levels_sequencer.h
// by Omar Cornut
//---------------------------------------------------------------------
// Game - Levels Sequencer
//---------------------------------------------------------------------

#define SEQUENCER_VAR_MAX	(4)
#define SEQUENCER_LABEL_MAX	(4)

//---------------------------------------------------------------------

#ifdef SEQUENCER_INSTRUCTIONS_NEEDED

//---------------------------------------------------------------------
// Sequence Instructions Codes
//---------------------------------------------------------------------
// Variables index: 0 to SEQUENCER_VAR_MAX-1
// Loop index: 0 to SEQUENCER_LABEL_MAX-1
// Note: <bar> and <type> are randomized if zero
//---------------------------------------------------------------------
#define SEQUENCE_DELAY		(0)		// Arg: <delay in 1/10 second>
#define SEQUENCE_ALIEN		(1)  	// Arg: <bar*>, <alien type*>
#define SEQUENCE_VAR_SET	(2)		// Arg: <var idx>, <value>
#define SEQUENCE_LABEL_SET	(3)		// Arg: <label idx>
#define SEQUENCE_DJNZ		(4)		// Arg: <var idx>, <label idx>
#define SEQUENCE_ENTERTAIN	(5)		// Arg: <length in 1/10 second>
#define SEQUENCE_WAIT_COUNT (6)		// Arg: <bar*>, <alien count>
#define SEQUENCE_END		(7)		// Arg: none
#define SEQUENCE_MAX		(8)
//---------------------------------------------------------------------
// Difficulty Level prefixes
//---------------------------------------------------------------------
#define EASY				(0x01)
#define NORMAL				(0x02)
#define HARD				(0x04)
//---------------------------------------------------------------------
// Parameters
//---------------------------------------------------------------------
#define	VAR(_IDX_)			(_IDX_)
#define LOOP(_IDX_)			(_IDX_)
#define BAR(_IDX_)			(1 << ((_IDX_) - 1))
#define BAR_ALL				(BAR(1) | BAR(2) | BAR(3) | BAR(4))
#define BAR_RANDOM			(BAR_ALL)
#define ALIEN(_IDX_)		(1 << ((_IDX_) - 1))
#define ALIEN_RANDOM		(ALIEN(1) | ALIEN(2) | ALIEN(3) | ALIEN(4))
//---------------------------------------------------------------------

#endif /* #ifdef SEQUENCER_INSTRUCTIONS_DEFINES */

//---------------------------------------------------------------------
// STRUCTURES
//---------------------------------------------------------------------

typedef struct
{
	u16					active;							/* Sequencer is active	*/
	u16					completed;						/* Sequence is complete */
	u16					delay;							/* Current delay		*/
	u8 *				sequence;						/* Sequence				*/
	u8 *				ip;								/* Instruction Pointer 	*/
	u8					vars[SEQUENCER_VAR_MAX];		/* Variables 			*/
	u16					labels[SEQUENCER_LABEL_MAX];	/* Labels 				*/
}						t_level_sequencer;

//---------------------------------------------------------------------
// DATA
//---------------------------------------------------------------------

extern t_level_sequencer	sequencer;

//---------------------------------------------------------------------
// FUNCTIONS
//---------------------------------------------------------------------

void			level_sequencer_start(const u8 *sequence);
void			level_sequencer_stop(void);
void			level_sequencer_update(void);

//---------------------------------------------------------------------
