//---------------------------------------------------------------------
// SPACE TAPPER: game_player.h
// by Omar Cornut
//---------------------------------------------------------------------
// Game - Player
//---------------------------------------------------------------------

// Player Palette index
#define PLAYER_PALETTE_IDX              	    	(0)
                        
// Player Animations            	        	
#define	PLAYER_ANIM_STANDING						(0)
#define PLAYER_ANIM_SWITCHING						(1)
#define PLAYER_ANIM_RUNNING							(2)
#define PLAYER_ANIM_SERVING							(3)
#define PLAYER_ANIM_SERVING_HOLD					(4)		// FIXME: rename to UP
#define PLAYER_ANIM_SERVING_SERVING					(5)		// FIXME: rename to DOWN
#define PLAYER_ANIM_THROWING						(6)
#define PLAYER_ANIM_BEER_BREAKING					(7)
#define PLAYER_ANIM_SLIDING							(8)
#define PLAYER_ANIM_BEER_CATCHING_IDLING			(9)
#define PLAYER_ANIM_BEER_CATCHING_SERVING_UP		(10)
#define PLAYER_ANIM_BEER_CATCHING_SERVING_DOWN		(11)
#define PLAYER_ANIM_RUNNING_SWITCH					(12)
#define PLAYER_ANIM_VICTORY							(13)

// Player Sprite informations
#define	PLAYER_SPR_SIZE_X							(32)
#define	PLAYER_SPR_SIZE_Y							(64)
#define PLAYER_SPR_SLIDING_SIZE_X					(64)
#define PLAYER_SPR_SLIDING_SIZE_Y					(32)
#define	PLAYER_SPR_OFFSET_Y							(-26)
#define PLAYER_SPR_HEAD_OFFSET_X					(16)
#define PLAYER_SPR_HEAD_OFFSET_Y					(22)

// Miscellaenous           	    	
#define PLAYER_SPEED								(FFIX4(2.0))	// Player speed (per frame)
#define PLAYER_BAR_START							(1)				// Starting bar (may go in leve)
#define PLAYER_SLIDING_SPEED						(FFIX4(2.0))
#define PLAYER_SLIDING_DELAY						(60)

// Player States Stack
#define PLAYER_STATE_STACK_SIZE						(2)

// Player States
// (those are actually not used as flags, but the bits-values are for quick testing)
#define PLAYER_STATE_IDLING							(0x0001)
#define PLAYER_STATE_SERVING						(0x0002)
#define PLAYER_STATE_RUNNING						(0x0004)
#define PLAYER_STATE_SWITCHING_UP					(0x0008)
#define PLAYER_STATE_SWITCHING_DOWN					(0x0010)
#define PLAYER_STATE_SLIDING						(0x0020)
#define PLAYER_STATE_BEER_BREAKING          		(0x0040)
#define PLAYER_STATE_BEER_CATCHING					(0x0080)
#define PLAYER_STATE_VICTORY						(0x0100)
                                                	
// Player Sub States
//#define PLAYER_SUB_STATE_SERVING_
#define PLAYER_SUB_STATE_SLIDING_SLIDING			(0)
#define PLAYER_SUB_STATE_SLIDING_CRASHING			(1)
#define PLAYER_SUB_STATE_SLIDING_DELAY				(2)
#define PLAYER_SUB_STATE_BEER_CATCHING_IDLING		(0)
#define PLAYER_SUB_STATE_BEER_CATCHING_SERVING		(1)

//---------------------------------------------------------------------

typedef struct
{
	fs16				xf;					/* X Position (0.0-240.0) in Fixed 12/4		*/
	fs16				yf;					/* Y Position (0.0-160.0) in Fixed 12/4		*/
	s16					xi;					/* X Position (0-240) copied from xf above	*/
	s16					yi;					/* Y Position (0-160) copied from yf above	*/
	u16					state;				/* Player state								*/
	u16					bar_idx;			/* Current bar the player is aligned with	*/
	u8					active;				/* Set if player is to be updated			*/	// FIXME: yet unused
	u8					sub_state;			/* Player sub-state							*/
	u8					state_custom_value;	/* Custom value used by state				*/
	u8					direction;			/* Player direction							*/
	u8					waiting_anim_loop;	/* Currently waiting for anim loop			*/
	t_animator_state	anim_state;			/* Anim state (ONLY VALID WHEN 'THIS' IS PUSHED) */
}						t_player_state;

typedef struct
{
	t_player_state *	state;				/* Current player state (point in _stack)	*/
	u16					sprite_idx;			/* Sprite index								*/
	t_sprite *			sprite;				/* Sprite (== &sprites[sprite_idx])			*/
	t_entity *			beer;				/* Currently attached beer (NULL if none)	*/
	t_entity *			beer_catched;		/* Currently catched beer (NULL if none)	*/
	t_animator			anim;				/* Animator for the player sprite			*/
	t_player_state		states_stack[PLAYER_STATE_STACK_SIZE];	/* Player states stack	*/
}						t_player;

extern t_player			player;

//---------------------------------------------------------------------

void	player_init_videodata(void);
void	player_init(void);
void	player_delete(void);
void	player_update(void);

void	player_state_slide(int bar_idx);
void	player_state_beer_catching(t_entity *beer);
void    player_state_beer_breaking(int bar_idx);

void	player_beer_delete(void);
void	player_beer_catched_delete(void);
void	player_beer_catched_catch(void);

void    player_live_lose(void);
void	player_victory(void);
void	player_victory_next_level(void);

void	player_state_push(void);
void	player_state_pop(void);

//---------------------------------------------------------------------
