//---------------------------------------------------------------------
// SPACE TAPPER: game_tappers.c
// by Omar Cornut
//---------------------------------------------------------------------
// Game - Tappers
//---------------------------------------------------------------------

#include "tapper.h"

//---------------------------------------------------------------------
// DATA
//---------------------------------------------------------------------

static int      tappers_tiles_start;
t_tapper        tappers[BAR_MAX];

//---------------------------------------------------------------------
// FUNCTIONS
//---------------------------------------------------------------------

void			tappers_init_videodata(void)
{
	palette_copy_16(game.state.level->layout->tappers_palette, MEM_PALx_OBJ(TAPPERS_PALETTE_IDX));
    tappers_tiles_start = vram_obj_copy(game.state.level->layout->tappers_tiles, 2 * 4 * TILE_SIZE_16 * 4 /* FIXME */);
}

void            tappers_init(void)
{
    int         i;
    
    for (i = 0; i < BAR_MAX; i++)
	{
        t_bar *bar = &bars[i];
        t_tapper *tapper = &tappers[i];

        // Setup state
        tapper->state = TAPPER_STATE_UP;

        // Create and Setup associated sprite
    	tapper->sprite_idx = sprite_new(ENTITY_SUB_PRIORITY_TAPPER(i));
    	sprites[tapper->sprite_idx].oam.attr0 = SPR_A0_COLOR_16 | SPR_A0_SHAPE_TALL | SPR_A0_Y_Set(bar->y_tapper);
    	sprites[tapper->sprite_idx].oam.attr1 = SPR_A1_SIZE_32 | SPR_A1_X_Set(bar->x_tapper) | (bar->direction ? SPR_A1_FLIP_H : SPR_A1_FLIP_NONE);
    	sprites[tapper->sprite_idx].oam.attr2 = SPR_A2_PRIORITY_Set(ENTITY_SPR_PRIORITY(i+1)) | SPR_A2_PALETTE_Set(TAPPERS_PALETTE_IDX) | SPR_A2_CHAR_Set(tappers_tiles_start);

    	// Setup Animation
    	animator_init(&tapper->anim, (ts_anims_set *)data_tapper_anim_set, &sprites[tapper->sprite_idx],
				        ANIMATOR_MODE_OAM_CHANGE_IDX, (void *)(u32)tappers_tiles_start, NULL);
	    animator_start(&tapper->anim, TAPPERS_ANIM_UP, 0);
    }
}

//---------------------------------------------------------------------

void        tappers_update(void)
{
    int     i;
    
    for (i = 0; i < BAR_MAX; i++)
        animator_update(&tappers[i].anim);
}

//---------------------------------------------------------------------

void		tappers_sprites_move_y(int y_offset)
{
	int		i;
	
	for (i = 0; i < BAR_MAX; i++)
    	sprites[tappers[i].sprite_idx].oam.y += y_offset;
}

//---------------------------------------------------------------------

void        tapper_up(int bar_idx)
{
    tappers[bar_idx].state = TAPPER_STATE_UP;
    animator_start(&tappers[bar_idx].anim, TAPPERS_ANIM_UP, 0);
}

void        tapper_down(int bar_idx)
{
    tappers[bar_idx].state = TAPPER_STATE_DOWN;
    animator_start(&tappers[bar_idx].anim, TAPPERS_ANIM_DOWN, 0);
}

void        tapper_down_immediate(int bar_idx)
{
    tappers[bar_idx].state = TAPPER_STATE_DOWN;
    animator_start(&tappers[bar_idx].anim, TAPPERS_ANIM_DOWN_IMMEDIATE, 0);
}

//---------------------------------------------------------------------
