//---------------------------------------------------------------------
// GBALIB: dma.h
// by Omar Cornut
//---------------------------------------------------------------------
// DMA System
//---------------------------------------------------------------------
// Simple wrapper around hardware DMA.
//---------------------------------------------------------------------

#include "gbalib.h"

// FUNCTION: CPU_Naive_Memory_Copy()
// Copy memory naively from any location to another.
// NOTE: this function write per 4-bytes blocks, then byte per byte, be careful...
void	CPU_Naive_Memory_Copy(void *dst, void *src, u32 size)
{
	// FIXME: profile the differents cases and correct depending on results
	while (size >= 16)
	{
		*((u32 *)dst)++ = *((u32 *)src)++;
		*((u32 *)dst)++ = *((u32 *)src)++;
		*((u32 *)dst)++ = *((u32 *)src)++;
		*((u32 *)dst)++ = *((u32 *)src)++;
		size -= 16;
	}
	while (size >= 4)
	{
		*((u32 *)dst)++ = *((u32 *)src)++;
		size -= 4;
	}
	while (size >= 1)
	{
		*((u8 *)dst)++ = *((u8 *)src)++;
		size--;
	}
}

// FUNCTION: DMA_Copy()
// Enable given	channel	DMA, with given	parameters,	source & destination addresses,	size
void	DMA_Copy(int channel, u32 mode,	void *src, void	*dst, u32 size)
{
	switch (channel)
	{
		case 0:	REG_DMA0SAD	= (u32)src;
				REG_DMA0DAD	= (u32)dst;
				REG_DMA0CNT	= size | mode;
				break;
		case 1:	REG_DMA1SAD	= (u32)src;
				REG_DMA1DAD	= (u32)dst;
				REG_DMA1CNT	= size | mode;
				break;
		case 2:	REG_DMA2SAD	= (u32)src;
				REG_DMA2DAD	= (u32)dst;
				REG_DMA2CNT	= size | mode;
				break;
		case 3:	REG_DMA3SAD	= (u32)src;
				REG_DMA3DAD	= (u32)dst;
				REG_DMA3CNT	= size | mode;
				break;
	}	
}
