//---------------------------------------------------------------------
// GBALIB: fixed.h
// by Omar Cornut
//---------------------------------------------------------------------
// Fixed Point values
//---------------------------------------------------------------------
// Some stuff borrowed from ARM application notes
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// FIXED TYPES
//---------------------------------------------------------------------

typedef	unsigned short		fu16;
typedef unsigned int		fu32;
typedef signed short		fs16;
typedef signed int			fs32;

//---------------------------------------------------------------------
// MACROS
//---------------------------------------------------------------------
// Convert to and from floating point
#define ITOFIX(d, q)		((int)((d) << (q)))
#define FTOFIX(d, q)		((int)((d) * (float)(1 << (q))))
#define TOFLOAT(a, q)		((float)(a) / (float)(1 << (q)))

#define	IFIX4(d)			ITOFIX(d, 4)
#define	FFIX4(d)			FTOFIX(d, 4)

// The basic operations performed on two numbers 'a' and 'b' of fixed
// point 'q' format returning the answer in 'q' format
#define FADD(a,b)			((a) + (b))
#define FSUB(a,b)			((a) - (b))
#define FMUL(a,b,q)			(((a) * (b)) >> (q))
#define FDIV(a,b,q)			(((a) << (q)) / (b))

// The basic operations where 'a' is of fixed point 'q' format and 'b' is
// an integer
#define FADDI(a,b,q)		((a) + ((b) << (q)))
#define FSUBI(a,b,q)		((a) - ((b) << (q)))
#define FMULI(a,b)			((a) * (b))
#define FDIVI(a,b)			((a) / (b))

// Convert 'a' from 'q1' format to 'q2' format
#define FCONV(a, q1, q2)	(((q2) > (q1)) ? (a)<<((q2)-(q1)) : (a)>>((q1)-(q2)))

// The general operation between 'a' in 'q1' format and 'b' in 'q2' format
// returning the result in 'q3' format
#define FADDG(a,b,q1,q2,q3)	(FCONV(a,q1,q3) + FCONV(b,q2,q3))
#define FSUBG(a,b,q1,q2,q3) (FCONV(a,q1,q3) - FCONV(b,q2,q3))
#define FMULG(a,b,q1,q2,q3) FCONV((a)*(b), (q1)+(q2), q3)
#define FDIVG(a,b,q1,q2,q3) (FCONV(a, q1, (q2)+(q3))/(b))

// Get the integer part of 'a' in 'q' format
#define FINT(a,q)			((a) >> (q))
#define FINT4(a)			FINT(a, 4)

// Get the integer part in 'q' format of 'a' in 'q' format
#define FTRUNC(a,q)			((a) & ~((1 << (q)) - 1))
#define FTRUNC4(a)			FTRUNC(a, 4)

// Get the decimal part of 'a' in 'q' format
#define FDEC(a,q)			((a) & ((1 << (q)) - 1))
#define FDEC4(a)			FDEC(a,4)

//---------------------------------------------------------------------
