//---------------------------------------------------------------------
// GBALIB: intrs.c
// by Omar Cornut
//---------------------------------------------------------------------
// Interrupts System
//---------------------------------------------------------------------
// Manage interrupts and interrupts handlers
//---------------------------------------------------------------------

#include "gbalib.h"

//---------------------------------------------------------------------

// [PRIVATE] DATA: intrs_handlers[]
// Pointers to user interrupt handler
static void	(*intrs_handlers[INTR_MAX])(void);

// [PRIVATE] FUNCTION: intrs_handler()
// Actual interrupt handler called by CPU on interrupt
// Note: loop is intentionnally unrolled in a certain order, to give priority to quick interrupts
static void	intrs_handler(void)
{
	u16		iflags;
	
	iflags = REG_IF;						// Read Interrupt Flags
	REG_IME = REG_IME_DISABLE;				// Disable Master Interrupts

	// Check H-Blank interrupt
	/*
	if (iflags & REG_IE_HBLANK)
	{
		intrs_handlers[INTR_HBLANK]();		// Call handler
		REG_IF = REG_IE_HBLANK;				// Disable H-Blank Interrupt Flags
		REG_IME = REG_IME_ENABLE;			// Re-enable Master Interrupts
		return;
	}
	*/

	// Check V-Count interrupt
	if (iflags & REG_IE_VCOUNT)
	{
		intrs_handlers[INTR_VCOUNT]();		// Call handler
		REG_IF = REG_IE_VCOUNT;				// Disable V-Count Interrupt Flags
		REG_IME = REG_IME_ENABLE;			// Re-enable Master Interrupts
		return;
	}
	
	// Check for other interrupts
/*
	int i;
	for (i = 0; i < INTR_MAX; i++)			// For each kind of interrupt, check its flags and call handler
		if (iflags & (1 << i))
			if (intrs_handlers[i])
				intrs_handlers[i]();
*/

	REG_IF = iflags;						// Clear Interrupt Flags (the register needs '1' bits to be written)
	REG_IME = REG_IME_ENABLE;				// Re-enable Master Interrupts
}

// FUNCTION: intrs_init()
// Initialize interrupt system
void		intrs_init(void)
{
	int		i;
	
	REG_INT_HANDLER = (u32)intrs_handler;
	REG_IME = REG_IME_DISABLE;
	REG_IE = REG_IE_NONE;
	for (i = 0; i < INTR_MAX; i++)
		intrs_handlers[i] = NULL;
}

//---------------------------------------------------------------------

// FUNCTION: intrs_all_enable()
// Disable interrupts on CPU
void		intrs_cpu_enable(void)
{
	REG_IME = REG_IME_ENABLE;
}

// FUNCTION: intrs_all_disable()
// Disable interrupts on CPU
void		intrs_cpu_disable(void)
{
	REG_IME = REG_IME_DISABLE;
}

// FUNCTION: intrs_one_enable()
// Enable given interrupt type
void		intrs_one_enable(int intr_type)
{
	REG_IE |= (1 << intr_type);
}

// FUNCTION: intrs_one_disable()
// Disable given interrupt type
void		intrs_one_disable(int intr_type)
{
	REG_IE &= ~(1 << intr_type);
}

// FUNCTION: intrs_one_set_handler()
// Set user handler for given interrupt type
void		intrs_one_handler_set(int intr_type, void (*handler)(void))
{
	intrs_handlers[intr_type] = handler;
}

//---------------------------------------------------------------------
