//---------------------------------------------------------------------
// GBALIB: intrs.h
// by Omar Cornut
//---------------------------------------------------------------------
// Interrupts System
//---------------------------------------------------------------------
// Manage interrupts and interrupts handlers
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// DEFINITIONS
//---------------------------------------------------------------------

// Interrupts Types
#define INTR_VBLANK		(0)		// V-Blank Interrupt (thrown by VDP)
#define INTR_HBLANK		(1)		// H-Blank Interrupt (thrown by VDP)
#define INTR_VCOUNT		(2)		// V-Count Interrupt (thrown by VDP)
#define INTR_TM0		(3)		// Timer 0 Interrupt
#define INTR_TM1		(4)		// Timer 1 Interrupt
#define INTR_TM2		(5)		// Timer 2 Interrupt
#define INTR_TM3		(6)		// Timer 3 Interrupt
#define INTR_COMM		(7)		// Communication Port Interrupt
#define INTR_DMA0		(8)		// DMA 0 Interrupt
#define INTR_DMA1		(9)		// DMA 1 Interrupt
#define INTR_DMA2		(10)	// DMA 2 Interrupt
#define INTR_DMA3		(11)	// DMA 3 Interrupt
#define INTR_KEY		(12)	// Key Interrupt
#define INTR_GAMEPAK	(13)	// GamePak Interrupt (thrown by cartridge)
#define INTR_MAX		(14)

//---------------------------------------------------------------------
// FUNCTIONS
//---------------------------------------------------------------------

// FUNCTION: intrs_init()
// Initialize interrupt system
void	intrs_init(void);

// FUNCTION: intrs_all_enable/disable()
// Enable/Disable interrupts on CPU
void	intrs_cpu_enable(void);
void	intrs_cpu_disable(void);

// FUNCTION: intrs_one_enable/disable()
// Enable/Disable given interrupt type
void	intrs_one_enable(int intr_type);
void	intrs_one_disable(int intr_type);

// FUNCTION: intrs_one_set_handler()
// Set user handler for given interrupt type
void	intrs_one_handler_set(int intr_type, void (*handler)(void));

//---------------------------------------------------------------------
