//---------------------------------------------------------------------
// GBALIB: vram.h
// by Omar Cornut
//---------------------------------------------------------------------
// VRAM Manager
//---------------------------------------------------------------------
// This is currently an extremely primitive VRAM Manager
// Things to do:
//  - Be able to free blocks of tiles (basically needs to write the VRAM Manager here..)
//  - Be able to specify non-usable memory area (used to store maps, etc..)
//	- Implement a data management system, via usage instance counter
//  - Perform benchs between DMA/16 and DMA/32 for copying to VRAM
//  - ...
//---------------------------------------------------------------------

//---------------------------------------------------------------------
// FUNCTIONS
//---------------------------------------------------------------------

// FUNCTION: vram_{bg,obj}_start()
// Setup subsequent copying at given location
void        vram_bg_start(u16 *addr);
void        vram_obj_start(u16 *addr);

// FUNCTION: vram_{bg,obj}_start()
// Allocate memory of given size (in bytes)
// Return starting tile number
u16         vram_bg_alloc(int size);
u16         vram_obj_alloc(int size);

// FUNCTION: vram_{bg,obj}_copy()
// Allocate memory of given size (in bytes), and copy data from given source
// Return starting tile number
u16         vram_bg_copy(u16 *src, int size);
u16         vram_obj_copy(u16 *src, int size);

// FUNCTION: vram_bg_copy_block()
// Allocate memory of given size (in bytes), and copy data from given source
// Then fill given block with tile number information, and given size information
void        vram_bg_copy_block(u16 *src, t_bg_block *block, int block_x, int block_y);
void        vram_bg_copy_block_256(u16 *src, t_bg_block *block, int block_x, int block_y);

// FUNCTION: vram_bg_copy_block_map_256()
static inline void  vram_bg_copy_block_map_256(t_bg_block_map *block, u16 *gfx_src, int gfx_size, u16 *map_src, int size_x, int size_y)
{
    u16 tiles_idx = vram_bg_copy(gfx_src, gfx_size);
    block->tiles_idx = tiles_idx / 2;
    block->map = map_src;
    block->size_x = size_x;
    block->size_y = size_y;
}

//---------------------------------------------------------------------
