//---------------------------------------------------------------------
// SPACE TAPPER: interface_game_over.c
// by Omar Cornut
//---------------------------------------------------------------------
// Interface - Game Over
//---------------------------------------------------------------------

#include "tapper.h"

//---------------------------------------------------------------------
// DATA
//---------------------------------------------------------------------

static int					step;
static u8					letters_sprite_idx[8];
const struct { u16 x, y; }	letters_final_pos[8] = 
{
	{  40, 126 }, // G
	{  57, 125 }, // A
	{  71, 128 }, // M
	{  85, 127 }, // E
	{ 116, 129 }, // O
	{ 130, 129 }, // V
	{ 141, 131 }, // E
	{ 153, 127 }  // R
};

//---------------------------------------------------------------------
// FUNCTIONS
//---------------------------------------------------------------------

void	    interface_game_over_start(va_list args)
{
	int		i;
	
	// Copy Palettes
	palette_copy((u16 *)gameover_Palette, MEM_PAL_BG + 3, 253);
	palette_copy((u16 *)gameover_letters_Palette, MEM_PAL_OBJ, 256);

	// Copy background Tiles using crap VRAM manager
	vram_bg_start(MEM_VRAM_BG_TILEx(64 * 2));
	vram_bg_copy((u16 *)gameover_Tiles, sizeof (gameover_Tiles));

	// Setup background 0 (layout)
	// -- Priority 3, 256 Colors, 32x32 tiles, Screen base block @ 0x6000C000, Character base block 0
	background_init(&game.bg0, 0, REG_BGxCNT_COLOR_256 | REG_BGxCNT_SCREEN_SIZE_256_256 | REG_BGxCNT_CHAR_BASE_Set(0) |
				  			 	  REG_BGxCNT_PRIORITY_Set(3) | REG_BGxCNT_SCREEN_BASE_Set(28));
	background_map_copy(&game.bg0, (u16 *)gameover_Map, 0);

	// Copy sprites Tiles using crap VRAM manager
	vram_obj_start(MEM_VRAM_OBJ_TILEx(0));
	vram_obj_copy((u16 *)gameover_letters_Bitmap, sizeof (gameover_letters_Bitmap));

    // Setup sprites for letters
    sprites_clear();
    for (i = 0; i < 8; i++)
    {
    	letters_sprite_idx[i] = sprite_new(1);
    	t_sprite *sprite = &sprites[letters_sprite_idx[i]];
    	sprite->oam.attr0 = SPR_A0_COLOR_256 | SPR_A0_SHAPE_SQUARE | SPR_A0_ROTATE | SPR_A0_Y_Set(130);
    	int x = RES_WIDTH + 8 + letters_final_pos[i].x - letters_final_pos[0].x + i * 18;
    	sprite->oam.attr1 = SPR_A1_SIZE_32 | SPR_A1_ROTIDX_Set(i) | SPR_A1_X_Set(x);
    	sprite->oam.attr2 = SPR_A2_PRIORITY_Set(0) | SPR_A2_CHAR_Set(i * 16*2);
    	sprite_oam_rotate(i, 0);
    }

	// Start at step 0 (window open)
	step = 0;
}

void	    interface_game_over_update(void)
{
	switch (step)
	{
		case 0: // Picture fade in
		{
			// Start Window Opening Effect
			section_call(SECTION_EFFECTS, EFFECT_P0_WINDOW_IN, 60, RES_WIDTH / 2, RES_HEIGHT / 2, RES_WIDTH / 2, RES_HEIGHT / 2);
			// Enable background 0 display
			REG_DISPCNT |= REG_DISPCNT_BG0_ENABLE | REG_DISPCNT_OBJ_ENABLE;
			step++;
			return;
		}
		case 1: // Check START key & Update movement
		{
			if (game.keys & REG_KEYINPUT_KEY_START)
				step++;
			int i;
			for (i = 0; i < 8; i++)
			{
    			t_sprite *sprite = &sprites[letters_sprite_idx[i]];
    			int d = letters_final_pos[i].y - sprite->oam.y;
    			if (d > 2) d = 2; else if (d < -2) d = -2;
    			if (d != 0)
    				sprite->oam.y += d;
				d = letters_final_pos[i].x - sprite->oam.x;
    			if (d > 2) d = 2; else if (d < -2) d = -2;
    			if (d != 0)
    				sprite->oam.x += d;
    			sprite_oam_rotate(i, (sprite->oam.x - letters_final_pos[i].x) & 0xFF);
			}
			//game.bg1.scroll_y ^= 1;
			//background_sync_scroll(&game.bg1);
			return;
		}
		case 2: // Fade out
		{
			effects_setup_closure(NO, NULL);
			section_call(SECTION_EFFECTS, EFFECT_P0_FADE_OUT, 32, REG_BLDCNT_TARGET1_BG0 | REG_BLDCNT_TARGET1_OBJ);
			step++;
			return;
		}
		case 3: // ...
		{
			section_jump(SECTION_INTERFACE_TITLE_SCREEN);
			REG_DISPCNT &= ~(REG_DISPCNT_BG0_ENABLE | REG_DISPCNT_OBJ_ENABLE);
			return;
		}
	}
}

void        interface_game_over_stop(void)
{
}

//---------------------------------------------------------------------

