//---------------------------------------------------------------------
// SPACE TAPPER: interface_title.c
// by Omar Cornut
//---------------------------------------------------------------------
// Interface - Title Screen
//---------------------------------------------------------------------

#include "tapper.h"
#define TITLE_NEW

//---------------------------------------------------------------------

#ifdef TITLE_NEW

//---------------------------------------------------------------------
// DATA
//---------------------------------------------------------------------

static int		    step;
static t_animator   anim_window;

//---------------------------------------------------------------------
// FUNCTIONS
//---------------------------------------------------------------------

void			interface_title_start(va_list args)
{
	t_bg_block_map	block_stars;
	
	// Copy Palette
	palette_copy((u16 *)title2_Palette, MEM_PAL_BG, 256);

	// Copy background Tiles using crap VRAM manager
	vram_bg_start(MEM_VRAM_BG_TILEx(64 * 2));
	vram_bg_copy((u16 *)title2_Tiles, sizeof (title2_Tiles));
	vram_bg_copy_block_map_256(&block_stars, (u16 *)title2_stars_Tiles, sizeof (title2_stars_Tiles), (u16 *)title2_stars_Map, 10, 10);

	// Setup background 0 (layout)
	// -- Priority 1, 256 Colors, 32x32 tiles, Screen base block @ 0x6000C000, Character base block 0
	background_init(&game.bg0, 0, REG_BGxCNT_COLOR_256 | REG_BGxCNT_SCREEN_SIZE_256_256 | REG_BGxCNT_CHAR_BASE_Set(0) |
				  			 	  REG_BGxCNT_PRIORITY_Set(1) | REG_BGxCNT_SCREEN_BASE_Set(28));
	background_map_copy(&game.bg0, (u16 *)title2_Map, 0);

	// Setup background 1 (window)
	// -- Priority 0, 256 Colors, 32x32 tiles, Screen base block @ 0x6000C800, Character base block 0
	background_init(&game.bg1, 1, REG_BGxCNT_COLOR_256 | REG_BGxCNT_SCREEN_SIZE_256_256 | REG_BGxCNT_CHAR_BASE_Set(0) |
								  REG_BGxCNT_PRIORITY_Set(0) | REG_BGxCNT_SCREEN_BASE_Set(29));
  	background_map_clear(&game.bg1, 0x0000);
  	game.bg1.scroll_x = -5;
  	game.bg1.scroll_y = -7;
  	background_sync_scroll(&game.bg1);

	// Setup window animation
    {
    t_bg_block block;
    vram_bg_copy_block_256((u16 *)title2_window_Bitmap, &block, 16, 8);
	animator_init(&anim_window, (ts_anims_set *)data_title2_window_anim_set, NULL,
				  ANIMATOR_MODE_VRAM_COPY_DATA, (u16 *)title2_window_Bitmap, MEM_VRAM_BG_TILEx(block.tiles_idx*2));
    animator_start(&anim_window, 0, 0);
    background_block_draw(&game.bg1, &block, 11, 10, BG_MAP_FLIP_NONE);
	}
	
	// Setup background 2 (stars)
	// -- Priority 3, 256 Colors, 32x32 tiles, Screen base block @ 0x6000C800, Character base block 0
	background_init(&game.bg2, 2, REG_BGxCNT_COLOR_256 | REG_BGxCNT_SCREEN_SIZE_256_256 | REG_BGxCNT_CHAR_BASE_Set(0) |
								  REG_BGxCNT_PRIORITY_Set(3) | REG_BGxCNT_SCREEN_BASE_Set(30));
  	background_map_clear(&game.bg2, 0x0000);
	background_block_map_draw(&game.bg2, &block_stars,  0,  0);
	background_block_map_draw(&game.bg2, &block_stars, 10,  0);
	background_block_map_draw(&game.bg2, &block_stars, 20,  0);
	background_block_map_draw(&game.bg2, &block_stars, 30,  0);
	
	// Setup Logo Transparency
	// REG_BLDCNT = REG_BLDCNT_TARGET1_BG1 | REG_BLDCNT_TARGET2_BG0 | REG_BLDCNT_EFFECT_ALPHA;
	// REG_BLDALPHA = REG_BLDALPHA_EVA_Set (14) | REG_BLDALPHA_EVB_Set (6);
	
	// Start at step 0 (fade in)
	step = 0;
}

void		interface_title_update(void)
{
	switch (step)
	{
		case 0: // Picture fade in
		{
			// Start fade in
			section_call(SECTION_EFFECTS, EFFECT_P0_FADE_IN, 32, REG_BLDCNT_TARGET1_BG0 | REG_BLDCNT_TARGET1_BG1 | REG_BLDCNT_TARGET1_BG2);
			// Enable background 0/1/2 display
			REG_DISPCNT |= REG_DISPCNT_BG0_ENABLE | REG_DISPCNT_BG1_ENABLE | REG_DISPCNT_BG2_ENABLE;
			step++;
			break;
		}
		case 1: // Update
		{
		    // Scroll stars
		    game.bg2.scroll_x--;
		    background_sync_scroll(&game.bg2);
            // Update animation
            animator_update(&anim_window);
		    // Check START key
			if (game.keys & REG_KEYINPUT_KEY_START)
				step++;
			break;
		}
		case 2: // Picture fade-out
		{
			effects_setup_closure(NO, NULL);
			section_call(SECTION_EFFECTS, EFFECT_P0_FADE_OUT, 32, REG_BLDCNT_TARGET1_BG0 | REG_BLDCNT_TARGET1_BG1 | REG_BLDCNT_TARGET1_BG2);
			step++;
			break;
		}
		case 3: // Jump to next Section
		{
			REG_DISPCNT &= ~(REG_DISPCNT_BG0_ENABLE | REG_DISPCNT_BG1_ENABLE | REG_DISPCNT_BG2_ENABLE);
			effects_setup_closure(YES, NULL);
			effects_stop();
			section_jump(SECTION_INTERFACE_LEVEL_SELECT);
			break;
		}
	}
	// Update Logo Transparency
	// logo_fading_value = (logo_fading_value + 1) & 0x3F;
	// if ((logo_fading_value & 0xF) == 0)
	//     REG_BLDALPHA = REG_BLDALPHA_EVA_Set (12 + (logo_fading_value + 1) / 0x10)  | REG_BLDALPHA_EVB_Set (6);
}

void        interface_title_stop(void)
{
	// Disable Logo Transparency Settings
	// REG_BLDCNT = 0x0000;
	// REG_BLDALPHA = 0x0000;
	font_init_videodata();
}

#else

//---------------------------------------------------------------------
// DATA
//---------------------------------------------------------------------

static int		step;

//---------------------------------------------------------------------
// FUNCTIONS
//---------------------------------------------------------------------

void			interface_title_start(va_list args)
{
	t_bg_block	block_title_logo;
	
	// Copy Palette
	palette_copy((u16 *)title_Palette, MEM_PAL_BG + 3, 253);

	// Copy background Tiles using crap VRAM manager
	vram_bg_start(MEM_VRAM_BG_TILEx(64 * 2));
	vram_bg_copy((u16 *)title_Tiles, sizeof (title_Tiles));
	vram_bg_copy_block_256((u16 *)title_logo_Bitmap, &block_title_logo, 11, 7);

	// Setup background 0 (layout)
	// -- Priority 3, 256 Colors, 32x32 tiles, Screen base block @ 0x6000C000, Character base block 0
	background_init(&game.bg0, 0, REG_BGxCNT_COLOR_256 | REG_BGxCNT_SCREEN_SIZE_256_256 | REG_BGxCNT_CHAR_BASE_Set(0) |
				  			 	  REG_BGxCNT_PRIORITY_Set(3) | REG_BGxCNT_SCREEN_BASE_Set(28));
	background_map_copy(&game.bg0, (u16 *)title_Map, 0);

	// Setup background 1 (logo)
	// -- Priority 1/2, 256 Colors, 32x32 tiles, Screen base block @ 0x6000C800, Character base block 0
	background_init(&game.bg1, 1, REG_BGxCNT_COLOR_256 | REG_BGxCNT_SCREEN_SIZE_256_256 | REG_BGxCNT_CHAR_BASE_Set(0) |
								  REG_BGxCNT_PRIORITY_Set(2) | REG_BGxCNT_SCREEN_BASE_Set(29));
  	background_map_clear(&game.bg1, 0x0000);
	background_block_draw(&game.bg1, &block_title_logo, 0, 13, BG_MAP_FLIP_NONE);
	
	// Setup Logo Transparency
	// REG_BLDCNT = REG_BLDCNT_TARGET1_BG1 | REG_BLDCNT_TARGET2_BG0 | REG_BLDCNT_EFFECT_ALPHA;
	// REG_BLDALPHA = REG_BLDALPHA_EVA_Set (14) | REG_BLDALPHA_EVB_Set (6);
	
	// Start at step 0 (fade in)
	step = 0;
}

void		interface_title_update(void)
{
	switch (step)
	{
		case 0: // Picture fade in
		{
			// Start fade in
			section_call(SECTION_EFFECTS, EFFECT_P0_FADE_IN, 32, REG_BLDCNT_TARGET1_BG0);
			// Enable background 0 display
			REG_DISPCNT |= REG_DISPCNT_BG0_ENABLE | REG_DISPCNT_BG1_ENABLE;
			step++;
			break;
		}
		case 1: // Check START key
		{
			// game.bg1.scroll_x ^= 1;
			// game.bg1.scroll_y ^= 1;
			// background_sync_scroll(&game.bg1);
			if (game.keys & REG_KEYINPUT_KEY_START)
				step++;
			break;
		}
		case 2: // Picture fade-out
		{
			effects_setup_closure(NO, NULL);
			section_call(SECTION_EFFECTS, EFFECT_P0_FADE_OUT, 32, REG_BLDCNT_TARGET1_BG0 | REG_BLDCNT_TARGET1_BG1);
			step++;
			break;
		}
		case 3: // Jump to next Section
		{
			REG_DISPCNT &= ~(REG_DISPCNT_BG0_ENABLE | REG_DISPCNT_BG1_ENABLE);
			effects_setup_closure(YES, NULL);
			effects_stop();
			section_jump(SECTION_INTERFACE_LEVEL_SELECT);
			break;
		}
	}
	// Update Logo Transparency
	// logo_fading_value = (logo_fading_value + 1) & 0x3F;
	// if ((logo_fading_value & 0xF) == 0)
	//     REG_BLDALPHA = REG_BLDALPHA_EVA_Set (12 + (logo_fading_value + 1) / 0x10)  | REG_BLDALPHA_EVB_Set (6);
}

void        interface_title_stop(void)
{
	// Disable Logo Transparency Settings
	// REG_BLDCNT = 0x0000;
	// REG_BLDALPHA = 0x0000;
}

#endif // TITLE_NEW

//---------------------------------------------------------------------
