//---------------------------------------------------------------------
// ANIM-To-C: animtoc.h
// by Omar Cornut, 2002
//---------------------------------------------------------------------
// Main Include File
//---------------------------------------------------------------------

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include "list.h"

//---------------------------------------------------------------------
// LEX & YACC STUFF
//---------------------------------------------------------------------

extern  char    *yytext;
extern  int     yylex(void);
extern  void    yyerror(char *s);
extern  int     yylineno;
extern  FILE *  yyin;
extern  int     yydebug;
extern  int     yyparse(void);

//---------------------------------------------------------------------
// BASIC TYPES
//---------------------------------------------------------------------

typedef     unsigned char   u8;
typedef     unsigned short  u16;
typedef     unsigned int    u32;
typedef     signed char     s8;
typedef     signed short    s16;
typedef     signed int      s32;

//---------------------------------------------------------------------
// STRUCTURES
//---------------------------------------------------------------------

typedef struct
{
	char *		source_file_name;
  	char *    	symbol_name;
  	u16       	picture_size_pixels_x;
  	u16       	picture_size_pixels_y;
  	u16       	picture_size_bytes;
  	u16       	picture_bpp;
  	// Positions
  	t_list *  	positions_data;
  	u32			positions_offset;
  	u32			positions_num;
  	// Bounding Boxes
  	t_list *  	boundingboxes_data;
  	u32			boundingboxes_offset;
  	u32			boundingboxes_num;
  	// Animations
  	t_list *  	anims_data;
  	u32			anims_offsets_offset;
  	u32			anims_offset;
  	u32			anims_num;
  	// Work
  	u32       	output_pos;
}           	t_anim_set;

typedef struct
{
    u32         pointer;
    u8          next_anim;
    u8          next_frame;
    t_list *    frames;
}               t_anim;

typedef struct
{               
    u8          delay;
    u8          i_picture;
    u8          i_offset;
    u8          i_boundingbox;
}               t_anim_frame;

typedef struct
{
    u16         index;
    s8          x;
    s8          y;
}               t_position;

#define ANIM_BOUNDINGBOX_SIZE			(4)	// x1 & x2
#define ANIM_BOUNDINGBOX_AXIS_TO_FLIP	(1)	// X (bounding box data will be flipped on X axis only)
#define ANIM_BOUNDINGBOX_FLIP_MAX		(1 << (ANIM_BOUNDINGBOX_AXIS_TO_FLIP))

typedef struct
{
    u16         index;
    s8          x1;
    s8          x2;
    //s8        y1;
    //s8        y2;
    int			size_x;
    int			size_y;
}               t_boundingbox;

//---------------------------------------------------------------------
// DATA
//---------------------------------------------------------------------

t_anim_set      anim_set;
t_anim *        anim_current;
t_anim_frame *  anim_frame_current;

//---------------------------------------------------------------------
// FUNCTIONS
//---------------------------------------------------------------------

void		animtoc_error(char *format, ...);
void        animtoc_fatal(char *format, ...);

//---------------------------------------------------------------------
