/**********************************************************************
** ANIM-To-C: animtoc.lex.l
** by Omar Cornut, 2002
***********************************************************************
** Lexer Definition
**********************************************************************/

/* Insert C declarations and LEX options there */

%{

#include "animtoc.h"
#include "animtoc.yac.h"

%}

%x COMMENT
%x STRING
%option noyywrap
%option yylineno

/* LEXICAL RULES */
%%

"Symbol"                { return TOK_GR_SYMBOL;                 }
"PictureSizePixels"     { return TOK_GR_PICTURE_SIZE_PIXELS;    }
"PictureSizeBytes"      { return TOK_GR_PICTURE_SIZE_BYTES;     }
"PictureBPP"            { return TOK_GR_PICTURE_BPP;            }
"Position"              { return TOK_GR_POSITION;               }
"BoundingBox"           { return TOK_GR_BOUNDINGBOX;            }
"Animation"             { return TOK_GR_ANIMATION;              }
"Frame"                 { return TOK_GR_FRAME;                  }
"Next"                  { return TOK_GR_NEXT;                   }
"End"                   { return TOK_GR_END;                    }

[_a-zA-Z][_a-zA-Z0-9]*  {
  yylval.v_string = strdup(yytext);
  return TOK_IDENT;
}

[ \t\r\n]+  {
  // return TOK_BLANK;
}

-?[0-9]+        {
  yylval.v_int = atoi(yytext);
  return TOK_INT;
}

"//" {
  BEGIN COMMENT;
}

<COMMENT>.* {
}

<COMMENT>\n {
  BEGIN 0;
}

<COMMENT><<EOF>> {
  printf("Unterminated comment, aborting.\n");
  exit(1);
}

"\\\n" {
}

\" {
  BEGIN STRING;
}

<STRING>[^\"]* {
  yylval.v_string = strdup(yytext);
  return TOK_STRING;
}

<STRING>[^\\]?\" {
  BEGIN 0;
}

<STRING><<EOF>> {
  printf("Unterminated string, aborting.\n");
  exit(1);
}

%%
/* Insert additionnal C code there */
