//---------------------------------------------------------------------
// ANIM-To-C: animtoc.yac.y
// by Omar Cornut, 2002
//---------------------------------------------------------------------
// Parser Definition
//---------------------------------------------------------------------

/* Insert C declarations and YACC options there */

%{

#include "animtoc.h"
#define YYERROR_VERBOSE

%}

%union {
  int   v_int;
  char *v_string;
}

%token              TOK_BLANK
%token <v_string>   TOK_IDENT TOK_STRING
%token <v_int>      TOK_INT
%token              TOK_GR_SYMBOL
%token              TOK_GR_PICTURE_SIZE_PIXELS TOK_GR_PICTURE_SIZE_BYTES TOK_GR_PICTURE_BPP
%token              TOK_GR_POSITION
%token              TOK_GR_BOUNDINGBOX
%token              TOK_GR_ANIMATION TOK_GR_FRAME TOK_GR_NEXT
%token              TOK_GR_END

/* GRAMMAR RULES */
%%

input   : /* Empty */
    | input line
    ;

line    : '\n'
    | TOK_BLANK { }
    | TOK_GR_END
        { 
            /* Ignore for the moment */
        }
    | TOK_GR_SYMBOL TOK_IDENT
        {
        	if (anim_set.symbol_name)
        		animtoc_fatal("SymbolName already defined!");
            anim_set.symbol_name = strdup($2);
        }
    | TOK_GR_PICTURE_SIZE_PIXELS TOK_INT TOK_INT
		{
			if (anim_set.picture_size_pixels_x || anim_set.picture_size_pixels_y)
				animtoc_fatal("PictureSizePixels already defined!");
			anim_set.picture_size_pixels_x = $2;
			anim_set.picture_size_pixels_y = $3;
    	}
    | TOK_GR_PICTURE_SIZE_BYTES TOK_INT 
        { 
        	if (anim_set.picture_size_bytes)
        		animtoc_fatal("PictureSizeBytes already defined!");
            anim_set.picture_size_bytes = $2;
        }
    | TOK_GR_PICTURE_BPP TOK_INT
    	{
    		if (anim_set.picture_bpp)
    			animtoc_fatal("PictureBPP already defined!");
    		anim_set.picture_bpp = $2;
    	}
    | TOK_GR_POSITION TOK_INT TOK_INT TOK_INT
        {
            t_position *position = malloc(sizeof (*position));
            position->index = $2;
            position->x = $3;
            position->y = $4;
            list_add_to_end(&anim_set.positions_data, position);
        }
    | TOK_GR_BOUNDINGBOX TOK_INT TOK_INT TOK_INT
    	{
    		t_boundingbox *bb = malloc(sizeof (*bb));
    		bb->index = $2;
    		bb->x1 = $3;
    		bb->x2 = $4;
    		//bb->y1 = $5;
    		//bb->y2 = $6;
    		bb->size_x = 0;
    		bb->size_y = 0;
    		list_add_to_end(&anim_set.boundingboxes_data, bb);
    	}
    | TOK_GR_BOUNDINGBOX TOK_INT TOK_INT TOK_INT TOK_INT TOK_INT
    	{
    		t_boundingbox *bb = malloc(sizeof (*bb));
    		bb->index = $2;
    		bb->x1 = $3;
    		bb->x2 = $4;
    		//bb->y1 = $5;
    		//bb->y2 = $6;
    		bb->size_x = $5;
    		bb->size_y = $6;
    		list_add_to_end(&anim_set.boundingboxes_data, bb);
    	}
    | TOK_GR_ANIMATION TOK_INT
        { 
            anim_current = malloc(sizeof (*anim_current));
            anim_current->pointer = anim_set.output_pos;
            list_add_to_end(&anim_set.anims_data, anim_current);
            anim_set.output_pos += 1;
        }
    | TOK_GR_FRAME TOK_INT TOK_INT 
        { 
            anim_frame_current = malloc(sizeof (*anim_frame_current));
            anim_frame_current->delay = $3;
            anim_frame_current->i_picture = $2;
            anim_frame_current->i_offset = 0;
            anim_frame_current->i_boundingbox = 0;
            list_add_to_end(&anim_current->frames, anim_frame_current);
            anim_set.output_pos += 4;
        }
    | TOK_GR_FRAME TOK_INT TOK_INT TOK_INT
        { 
            anim_frame_current = malloc(sizeof (*anim_frame_current));
            anim_frame_current->delay = $3;
            anim_frame_current->i_picture = $2;
            anim_frame_current->i_offset = $4;
            anim_frame_current->i_boundingbox = 0;
            list_add_to_end(&anim_current->frames, anim_frame_current);
            anim_set.output_pos += 4;
        }
    | TOK_GR_FRAME TOK_INT TOK_INT TOK_INT TOK_INT
        { 
            anim_frame_current = malloc(sizeof (*anim_frame_current));
            anim_frame_current->delay = $3;
            anim_frame_current->i_picture = $2;
            anim_frame_current->i_offset = $4;
            anim_frame_current->i_boundingbox = $5;
            list_add_to_end(&anim_current->frames, anim_frame_current);
            anim_set.output_pos += 4;
        }
    | TOK_GR_NEXT TOK_INT 
        { 
            anim_current->next_anim = $2;
            anim_current->next_frame = 0;
            anim_set.output_pos += 2;
        }
    | TOK_GR_NEXT TOK_INT TOK_INT
        {
            anim_current->next_anim = $2;
            anim_current->next_frame = $3;
            anim_set.output_pos += 2;
        }
    ;

%%

void    yyerror(char *s)
{
  fprintf(stderr, "line %d: %s\n", yylineno, s);
  exit(1);
}
